@@ -19,12 +19,6 @@ bootstrap_model <- function(model, iterations = 1000, verbose = FALSE, ...) {
   UseMethod("bootstrap_model")
 }
 
-#' @rdname bootstrap_model
-#' @export
-model_bootstrap <- bootstrap_model
-
-
-
 
 
 
@@ -87,7 +81,7 @@ bootstrap_model.merMod <- function(model, iterations = 1000, verbose = FALSE, ..
   }
 
   out <- as.data.frame(results$t)
-  names(out) <- insight::find_parameters(model)$conditional
+  names(out) <- insight::find_parameters(model, effects = "fixed")$conditional
 
   out
 }
