@@ -17,20 +17,44 @@
 #'   the de-meaned variable) by default.
 #'
 #' @details
+#'   \subsection{Heterogeneity Bias}{
+#'     Mixed models include different levels of sources of variability, i.e.
+#'     error terms at each level. When macro-indicators (or level-2 predictors,
+#'     or higher-level units, or more general: \emph{group-level predictors that
+#'     are \strong{constant} within groups}, such as "education" within participants,
+#'     or GDP within countries) are included as fixed effects (i.e. treated as
+#'     covariate at level-1), the variance that is left unaccounted for this covariate
+#'     will be absorbed into the error terms of level-1 and level-2. Hence, the error
+#'     terms will be correlated with the covariate, which violates one of the
+#'     assumptions of mixed models (iid, independent and identically distributed
+#'     error terms). This bias is also called the \emph{heterogeneity bias}
+#'     (\cite{Bell et al. 2015}). To resolve this problem, level-2 predictors
+#'     used as (level-1) covariates should be "group-meaned".
+#'   }
 #'   \subsection{Panel data and correlating fixed and group effects}{
 #'     \code{demean()} is intended to create group- and de-meaned variables
 #'     for panel regression models (fixed effects models), or for complex
-#'     random-effect-within-between models (see \cite{Bell et al. 2018}),
+#'     random-effect-within-between models (see \cite{Bell et al. 2015, 2018}),
 #'     where group-effects (random effects) and fixed effects correlate (see
-#'     \cite{Bafumi and Gelman 2006)}). This violation of one of the
-#'     \emph{Gauss-Markov-assumptions} can happen, for instance, when analysing panel
-#'     data. To control for correlating predictors and group effects, it is
-#'     recommended to include the group-meaned and de-meaned version of
-#'     \emph{time-varying covariates} in the model. By this, one can fit
-#'     complex multilevel models for panel data, including time-varying predictors,
-#'     time-invariant predictors and random effects. This approach is superior to
-#'     classic fixed-effects models, which lack information of variation in the
-#'     group-effects or between-subject effects.
+#'     \cite{Bafumi and Gelman 2006}). This can happen, for instance, when
+#'     analyzing panel data, which can lead to \emph{Heterogeneity Bias}. To
+#'     control for correlating predictors and group effects, it is recommended
+#'     to include the group-meaned and de-meaned version of \emph{time-varying covariates}
+#'     (and group-meaned version of \emph{time-invariant covariates} that are on
+#'     a higher level, e.g. level-2 predictors) in the model. By this, one can
+#'     fit complex multilevel models for panel data, including time-varying
+#'     predictors, time-invariant predictors and random effects.
+#'    }
+#'   \subsection{Why mixed models are preferred over fixed effects models}{
+#'     A mixed models approach including time-varying and time-constant fixed
+#'     effects as well as random effects is superior to classic fixed-effects
+#'     models, which lack information of variation in the group-effects or
+#'     between-subject effects. Furthermore, fixed effects regression cannot
+#'     include random slopes, which means that fixed effects regressions are
+#'     neglecting \dQuote{cross-cluster differences in the effects of lower-level
+#'     controls (which) reduces the precision of estimated context effects,
+#'     resulting in unnecessarily wide confidence intervals and low statistical
+#'     power} (\cite{Heisig et al. 2017}).
 #'   }
 #'   \subsection{Terminology}{
 #'     The group-meaned variable is simply the mean of an independent variable
@@ -56,6 +80,14 @@
 #'     categorical time-varying predictors to numeric to compute the de- and
 #'     group-meaned versions for these variables.
 #'   }
+#'   \subsection{De-meaning of factors with more than 2 levels}{
+#'     Factors with more than two levels are demeaned in two ways: first, these
+#'     are also converted to numeric and de-meaned; second, dummy variables
+#'     are created (binary, with 0/1 coding for each level) and these binary
+#'     dummy-variables are de-meaned in the same way (as described above).
+#'     Packages like \pkg{panelr} internally convert factors to dummies before
+#'     demeaning, so this behaviour can be mimicked here.
+#'   }
 #'   \subsection{De-meaning interaction terms}{
 #'     There are multiple ways to deal with interaction terms of within- and
 #'     between-effects. A classical approach is to simply use the product
@@ -88,7 +120,9 @@
 #' @references \itemize{
 #'   \item Bafumi J, Gelman A. 2006. Fitting Multilevel Models When Predictors and Group Effects Correlate. In. Philadelphia, PA: Annual meeting of the American Political Science Association.
 #'   \item Bell A, Fairbrother M, Jones K. 2018. Fixed and Random Effects Models: Making an Informed Choice. Quality & Quantity.
-#'   \item Giesselmann M, Schmidt-Catran A. (2018). Interactions in fixed effects regression models (Discussion Papers of DIW Berlin No. 1748). DIW Berlin, German Institute for Economic Research. Retrieved from https://ideas.repec.org/p/diw/diwwpp/dp1748.html
+#'   \item Bell A, Jones K. 2015. Explaining Fixed Effects: Random Effects Modeling of Time-Series Cross-Sectional and Panel Data. Political Science Research and Methods, 3(1), 133–153.
+#'   \item Giesselmann M, Schmidt-Catran A. 2018. Interactions in fixed effects regression models (Discussion Papers of DIW Berlin No. 1748). DIW Berlin, German Institute for Economic Research. Retrieved from https://ideas.repec.org/p/diw/diwwpp/dp1748.html
+#'   \item Heisig JP, Schaeffer M, Giesecke J. 2017. The Costs of Simplicity: Why Multilevel Models May Benefit from Accounting for Cross-Cluster Differences in the Effects of Controls. American Sociological Review 82 (4): 796–827.
 #'   \item Hoffman L. 2015. Longitudinal analysis: modeling within-person fluctuation and change. New York: Routledge
 #' }
 
@@ -149,12 +183,32 @@ demean <- function(x, select, group, suffix_demean = "_within", suffix_groupmean
   # find categorical predictors that are coded as factors
   categorical_predictors <- sapply(dat[select], is.factor)
 
-  # convert binrary predictors to numeric
+  # convert binary predictors to numeric
   if (any(categorical_predictors)) {
+    # convert categorical to numeric, and then demean
     dat[select[categorical_predictors]] <- lapply(
       dat[select[categorical_predictors]],
       function(i) as.numeric(i) - 1
     )
+    # convert categorical to dummy, and demean each binary dummy
+    for (i in select[categorical_predictors]) {
+      if (nlevels(x[[i]]) > 2) {
+        for (j in levels(x[[i]])) {
+          # create vector with zeros
+          f <- rep(0, nrow(x))
+          # for each matching level, set dummy to 1
+          f[x[[i]] == j] <- 1
+          dummy <- data.frame(f)
+          # colnames = variable name + factor level
+          # also add new dummy variables to "select"
+          colnames(dummy) <- sprintf("%s_%s", i, j)
+          select <- c(select, sprintf("%s_%s", i, j))
+          # add to data
+          dat <- cbind(dat, dummy)
+        }
+      }
+    }
+    # tell user...
     insight::print_color(
       sprintf(
         "Categorical predictors (%s) have been coerced to numeric values to compute de- and group-meaned variables.\n",
@@ -190,5 +244,14 @@ demean <- function(x, select, group, suffix_demean = "_within", suffix_groupmean
   colnames(x_dm) <- sprintf("%s%s", colnames(x_dm), suffix_demean)
   colnames(x_gm) <- sprintf("%s%s", colnames(x_gm), suffix_groupmean)
 
+  x_dm[] <- lapply(x_dm, function(i) {
+    attr(i, "within-effect") <- TRUE
+    i
+  })
+  x_gm[] <- lapply(x_gm, function(i) {
+    attr(i, "between-effect") <- TRUE
+    i
+  })
+
   cbind(x_gm, x_dm)
 }
