@@ -2,12 +2,34 @@
 
 
 #' @importFrom insight get_statistic get_parameters
-#' @importFrom stats confint
+#' @importFrom stats confint p.adjust.methods p.adjust
 #' @keywords internal
-.extract_parameters_generic <- function(model, ci, component, merge_by = c("Parameter", "Component"), standardize = NULL, effects = "fixed", robust = FALSE, ...) {
+.extract_parameters_generic <- function(model, ci, component, merge_by = c("Parameter", "Component"), standardize = NULL, effects = "fixed", robust = FALSE, df_method = NULL, p_adjust = NULL, ...) {
+  # check if standardization is required and package available
+  if (!is.null(standardize) && !requireNamespace("effectsize", quietly = TRUE)) {
+    insight::print_color("Package 'effectsize' required to calculate standardized coefficients. Please install it.\n", "red")
+    standardize <- NULL
+  }
+
+  # for refit, we completely refit the model, than extract parameters, ci etc. as usual
+  if (!is.null(standardize) && standardize == "refit") {
+    model <- effectsize::standardize(model, verbose = FALSE)
+    standardize <- NULL
+  }
+
   parameters <- insight::get_parameters(model, effects = effects, component = component)
   statistic <- insight::get_statistic(model, component = component)
-  robust <- if (isTRUE(robust)) "robust" else NULL
+
+  # check if we really have a component column
+  if (!("Component" %in% names(parameters)) && "Component" %in% merge_by) {
+    merge_by <- setdiff(merge_by, "Component")
+  }
+
+  # check Degrees of freedom
+  if (!.dof_method_ok(model, df_method)) {
+    df_method <- NULL
+  }
+
 
   # clean parameter names
 
@@ -20,29 +42,22 @@
   # column name for coefficients, non-standardized
   coef_col <- "Coefficient"
 
-  # Std Coefficients
+  # Std Coefficients for other methods than "refit"
   if (!is.null(standardize)) {
-    if (!requireNamespace("effectsize", quietly = TRUE)) {
-      insight::print_color("Package 'effectsize' required to calculate standardized coefficients. Please install it.\n", "red")
+    # standardize model parameters and calculate related CI and SE
+    std_coef <- effectsize::standardize_parameters(model, method = standardize)
+    parameters <- merge(parameters, std_coef, by = merge_by)
+    coef_col <- "Std_Coefficient"
+    # merge all data, including CI and SE for std. parameters
+    if (inherits(std_coef, "effectsize_std_params")) {
+      parameters <- merge(parameters, ci(std_coef, ci = ci), by = merge_by)
+      parameters <- merge(parameters, standard_error(std_coef), by = merge_by)
+    }
+    # if we have CIs, remember columns names to select later
+    if (!is.null(ci)) {
+      ci_cols <- c("CI_low", "CI_high")
     } else {
-      # standardize model parameters and calculate related CI and SE
-      std_coef <- effectsize::standardize_parameters(model, method = standardize)
-      parameters <- merge(parameters, std_coef, by = merge_by)
-      coef_col <- "Std_Coefficient"
-      # merge all data, including CI and SE for std. parameters
-      if (standardize == "refit") {
-        parameters <- merge(parameters, .ci_from_refit(std_coef, ci), by = merge_by)
-        parameters <- merge(parameters, attributes(std_coef)$standard_error, by = merge_by)
-      } else if (inherits(std_coef, "effectsize_std_params")) {
-        parameters <- merge(parameters, ci(std_coef, ci = ci), by = merge_by)
-        parameters <- merge(parameters, standard_error(std_coef), by = merge_by)
-      }
-      # if we have CIs, remember columns names to select later
-      if (!is.null(ci)) {
-        ci_cols <- c("CI_low", "CI_high")
-      } else {
-        ci_cols <- c()
-      }
+      ci_cols <- c()
     }
   }
 
@@ -50,7 +65,13 @@
   # CI - only if we don't already have CI for std. parameters
   if (is.null(standardize)) {
     if (!is.null(ci)) {
-      ci_df <- suppressMessages(ci(model, ci = ci, effects = effects, component = component, method = robust))
+      if (isTRUE(robust)) {
+        ci_df <- suppressMessages(ci_robust(model, ci = ci, ...))
+      } else if (!is.null(df_method)) {
+        ci_df <- suppressMessages(ci(model, ci = ci, effects = effects, component = component, method = df_method))
+      } else {
+        ci_df <- suppressMessages(ci(model, ci = ci, effects = effects, component = component))
+      }
       if (length(ci) > 1) ci_df <- bayestestR::reshape_ci(ci_df)
       ci_cols <- names(ci_df)[!names(ci_df) %in% c("CI", merge_by)]
       parameters <- merge(parameters, ci_df, by = merge_by)
@@ -61,21 +82,46 @@
 
 
   # p value
-  parameters <- merge(parameters, p_value(model, effects = effects, component = component, method = robust), by = merge_by)
+  if (isTRUE(robust)) {
+    parameters <- merge(parameters, p_value_robust(model, ...), by = merge_by)
+  } else if (!is.null(df_method)) {
+    parameters <- merge(parameters, p_value(model, effects = effects, component = component, method = df_method), by = merge_by)
+  } else {
+    parameters <- merge(parameters, p_value(model, effects = effects, component = component), by = merge_by)
+  }
+
 
   # standard error - only if we don't already have SE for std. parameters
-  if (is.null(standardize)) parameters <- merge(parameters, standard_error(model, effects = effects, component = component, method = robust), by = merge_by)
+  if (is.null(standardize)) {
+    if (isTRUE(robust)) {
+      parameters <- merge(parameters, standard_error_robust(model, ...), by = merge_by)
+    } else if (!is.null(df_method)) {
+      parameters <- merge(parameters, standard_error(model, effects = effects, component = component, method = df_method), by = merge_by)
+    } else {
+      parameters <- merge(parameters, standard_error(model, effects = effects, component = component), by = merge_by)
+    }
+  }
+
 
   # test statistic - fix values for robust estimation
-  if (!is.null(robust) && robust == "robust") statistic$Statistic <- parameters$Estimate / parameters$SE
-  parameters <- merge(parameters, statistic, by = merge_by)
+  if (isTRUE(robust)) {
+    parameters$Statistic <- parameters$Estimate / parameters$SE
+  } else {
+    parameters <- merge(parameters, statistic, by = merge_by)
+  }
+
 
   # dof
-  df_error <- degrees_of_freedom(model, method = "any")
+  if (!is.null(df_method)) {
+    df_error <- degrees_of_freedom(model, method = df_method)
+  } else {
+    df_error <- degrees_of_freedom(model, method = "any")
+  }
   if (!is.null(df_error) && (length(df_error) == 1 || length(df_error) == nrow(parameters))) {
     parameters$df_error <- df_error
   }
 
+
   # Rematch order after merging
   parameters <- parameters[match(original_order, parameters$.id), ]
 
@@ -91,6 +137,11 @@
   if (length(unique(parameters$Component)) == 1) parameters$Component <- NULL
   if (length(unique(parameters$Effects)) == 1 || effects == "fixed") parameters$Effects <- NULL
 
+  # adjust p-values?
+  if (!is.null(p_adjust) && tolower(p_adjust) %in% stats::p.adjust.methods && "p" %in% colnames(parameters)) {
+    parameters$p <- stats::p.adjust(parameters$p, method = p_adjust)
+  }
+
   rownames(parameters) <- NULL
   parameters
 }
@@ -103,11 +154,32 @@
 
 #' @importFrom stats confint
 #' @keywords internal
-.extract_parameters_mixed <- function(model, ci = .95, df_method = "wald", standardize = NULL, ...) {
-  parameters <- as.data.frame(summary(model)$coefficients, stringsAsFactors = FALSE)
-  parameters$Parameter <- row.names(parameters)
+.extract_parameters_mixed <- function(model, ci = .95, df_method = "wald", standardize = NULL, robust = FALSE, p_adjust = NULL, ...) {
+  # check if standardization is required and package available
+  if (!is.null(standardize) && !requireNamespace("effectsize", quietly = TRUE)) {
+    insight::print_color("Package 'effectsize' required to calculate standardized coefficients. Please install it.\n", "red")
+    standardize <- NULL
+  }
+
+  # for refit, we completely refit the model, than extract parameters,
+  # ci etc. as usual - therefor, we set "standardize" to NULL
+  if (!is.null(standardize) && standardize == "refit") {
+    model <- effectsize::standardize(model, verbose = FALSE)
+    standardize <- NULL
+  }
+
+  special_df_methods <- c("betwithin", "satterthwaite", "ml1", "kenward", "kr")
+
+  # get parameters and statistic
+  parameters <- insight::get_parameters(model, effects = "fixed", component = "all")
+  statistic <- insight::get_statistic(model, component = "all")
+
+  # sometimes, due to merge(), row-order messes up, so we save this here
   original_order <- parameters$.id <- 1:nrow(parameters)
 
+  # remove SE column
+  parameters <- .remove_columns(parameters, c("SE", "Std. Error"))
+
   # column name for coefficients, non-standardized
   coef_col <- "Coefficient"
 
@@ -119,32 +191,31 @@
     df <- Inf
   }
 
+  df_error <- data.frame(
+    Parameter = parameters$Parameter,
+    df_error = as.vector(df),
+    stringsAsFactors = FALSE
+  )
+  # for KR-dof, we have the SE as well, to save computation time
+  df_error$SE <- attr(df, "se", exact = TRUE)
 
-  # Std Coefficients
+
+  # Std Coefficients for other methods than "refit"
   if (!is.null(standardize)) {
-    if (!requireNamespace("effectsize", quietly = TRUE)) {
-      insight::print_color("Package 'effectsize' required to calculate standardized coefficients. Please install it.\n", "red")
+    # standardize model parameters and calculate related CI and SE
+    std_coef <- effectsize::standardize_parameters(model, method = standardize)
+    parameters <- merge(parameters, std_coef, by = "Parameter")
+    coef_col <- "Std_Coefficient"
+    # merge all data, including CI and SE for std. parameters
+    if (inherits(std_coef, "effectsize_std_params")) {
+      parameters <- merge(parameters, ci(std_coef, ci = ci), by = "Parameter")
+      parameters <- merge(parameters, standard_error(std_coef), by = "Parameter")
+    }
+    # if we have CIs, remember columns names to select later
+    if (!is.null(ci)) {
+      ci_cols <- c("CI_low", "CI_high")
     } else {
-      # remove SE column
-      parameters[["Std. Error"]] <- NULL
-      # standardize model parameters and calculate related CI and SE
-      std_coef <- effectsize::standardize_parameters(model, method = standardize)
-      parameters <- merge(parameters, std_coef, by = "Parameter")
-      coef_col <- "Std_Coefficient"
-      # merge all data, including CI and SE for std. parameters
-      if (standardize == "refit") {
-        parameters <- merge(parameters, .ci_from_refit(std_coef, ci), by = "Parameter")
-        parameters <- merge(parameters, attributes(std_coef)$standard_error, by = "Parameter")
-      } else if (inherits(std_coef, "effectsize_std_params")) {
-        parameters <- merge(parameters, ci(std_coef), by = "Parameter")
-        parameters <- merge(parameters, standard_error(std_coef), by = "Parameter")
-      }
-      # if we have CIs, remember columns names to select later
-      if (!is.null(ci)) {
-        ci_cols <- c("CI_low", "CI_high")
-      } else {
-        ci_cols <- c()
-      }
+      ci_cols <- c()
     }
   }
 
@@ -152,7 +223,14 @@
   # CI - only if we don't already have CI for std. parameters
   if (is.null(standardize)) {
     if (!is.null(ci)) {
-      ci_df <- ci_wald(model, ci = ci, dof = df)
+      if (isTRUE(robust)) {
+        ci_df <- suppressMessages(ci_robust(model, ci = ci, ...))
+      } else if (df_method %in% c("kenward", "kr")) {
+        # special handling for KR-CIs, where we already have computed SE
+        ci_df <- .ci_kenward_dof(model, ci = ci, df_kr = df_error)
+      } else {
+        ci_df <- ci(model, ci = ci, method = df_method, effects = "fixed")
+      }
       if (length(ci) > 1) ci_df <- bayestestR::reshape_ci(ci_df)
       ci_cols <- names(ci_df)[!names(ci_df) %in% c("CI", "Parameter")]
       parameters <- merge(parameters, ci_df, by = "Parameter")
@@ -162,39 +240,58 @@
   }
 
 
+  # standard error - only if we don't already have SE for std. parameters
+  if (is.null(standardize) || !("SE" %in% colnames(parameters))) {
+    if (isTRUE(robust)) {
+      parameters <- merge(parameters, standard_error_robust(model, ...), by = "Parameter")
+      # special handling for KR-SEs, which we already have computed from dof
+    } else if ("SE" %in% colnames(df_error)) {
+      se_kr <- df_error
+      se_kr$df_error <- NULL
+      parameters <- merge(parameters, se_kr, by = "Parameter")
+    } else {
+      parameters <- merge(parameters, standard_error(model, method = df_method, effects = "fixed"), by = "Parameter")
+    }
+  }
+
+
   # p value
-  if ("Pr(>|z|)" %in% names(parameters)) {
-    names(parameters)[grepl("Pr(>|z|)", names(parameters), fixed = TRUE)] <- "p"
+  if (isTRUE(robust)) {
+    parameters <- merge(parameters, p_value_robust(model, ...), by = "Parameter")
   } else {
-    parameters <- merge(parameters, p_value(model, dof = df), by = "Parameter")
+    if ("Pr(>|z|)" %in% names(parameters)) {
+      names(parameters)[grepl("Pr(>|z|)", names(parameters), fixed = TRUE)] <- "p"
+    } else if (df_method %in% special_df_methods) {
+      # special handling for KR-p, which we already have computed from dof
+      parameters <- merge(parameters, .p_value_dof_kr(model, params = parameters, dof = df_error), by = "Parameter")
+    } else {
+      parameters <- merge(parameters, p_value(model, dof = df, effects = "fixed"), by = "Parameter")
+    }
   }
 
 
   # adjust standard errors and test-statistic as well
-  if (is.null(standardize) && df_method %in% c("ml1", "satterthwaite", "kenward")) {
-    parameters[["Std. Error"]] <- NULL
-
-    if (df_method == "kenward") {
-      parameters <- merge(parameters, se_kenward(model), by = "Parameter")
-    } else if (df_method == "satterthwaite") {
-      parameters <- merge(parameters, se_satterthwaite(model), by = "Parameter")
-    } else {
-      parameters <- merge(parameters, se_ml1(model), by = "Parameter")
-    }
-
-    for (test_statistic in c("t value", "z value")) {
-      if (test_statistic %in% colnames(parameters)) {
-        parameters[[test_statistic]] <- parameters[["Estimate"]] / parameters[["SE"]]
-      }
-    }
+  if (!isTRUE(robust) && is.null(standardize) && df_method %in% special_df_methods) {
+    parameters$Statistic <- parameters$Estimate / parameters$SE
+  } else {
+    parameters <- merge(parameters, statistic, by = "Parameter")
   }
 
 
   # dof
-  if (!(df_method %in% c("ml1", "satterthwaite", "kenward")) && !"df" %in% names(parameters)) {
-    df_error <- degrees_of_freedom(model, method = "any")
-    if (!is.null(df_error) && (length(df_error) == 1 || length(df_error) == nrow(parameters))) {
-      parameters$df_error <- df_error
+  if (!"df" %in% names(parameters)) {
+    if (!df_method %in% special_df_methods) {
+      df_error <- data.frame(
+        Parameter = parameters$Parameter,
+        df_error = degrees_of_freedom(model, method = "any"),
+        stringsAsFactors = FALSE
+      )
+    }
+    if (!is.null(df_error) && nrow(df_error) == nrow(parameters)) {
+      if ("SE" %in% colnames(df_error)) {
+        df_error$SE <- NULL
+      }
+      parameters <- merge(parameters, df_error, by = "Parameter")
     }
   }
 
@@ -203,6 +300,7 @@
   parameters <- parameters[match(original_order, parameters$.id), ]
 
   # Renaming
+  names(parameters) <- gsub("Statistic", gsub("-statistic", "", attr(statistic, "statistic", exact = TRUE), fixed = TRUE), names(parameters))
   names(parameters) <- gsub("Std. Error", "SE", names(parameters))
   names(parameters) <- gsub("Estimate", "Coefficient", names(parameters))
   names(parameters) <- gsub("t value", "t", names(parameters))
@@ -212,12 +310,74 @@
   order <- c("Parameter", coef_col, "SE", ci_cols, "t", "z", "df", "df_error", "p")
   parameters <- parameters[order[order %in% names(parameters)]]
 
+  # adjust p-values?
+  if (!is.null(p_adjust) && tolower(p_adjust) %in% stats::p.adjust.methods && "p" %in% colnames(parameters)) {
+    parameters$p <- stats::p.adjust(parameters$p, method = p_adjust)
+  }
+
+  # if we have within/between effects (from demean()), we can add a component
+  # column for nicer printing...
+  parameters <- .add_within_between_effects(model, parameters)
+
   rownames(parameters) <- NULL
   parameters
 }
 
 
 
+.add_within_between_effects <- function(model, parameters) {
+
+  # This function checks whether the model contains predictors that were
+  # "demeaned" using the "demean()" function. If so, these columns have an
+  # attribute indicating the within or between effect, and in such cases,
+  # this effect is used as "Component" value. by this, we get a nicer print
+  # for model parameters...
+
+  parameters$Component <- "rewb-contextual"
+
+  within_effects <- .find_within_between(model, "within-effect")
+  if (!is.null(within_effects)) {
+    index <- unique(unlist(sapply(within_effects, function(i) {
+      grep(i, parameters$Parameter, fixed = TRUE)
+    })))
+    parameters$Component[index] <- "within"
+  }
+
+  between_effects <- .find_within_between(model, "between-effect")
+  if (!is.null(between_effects)) {
+    index <- unique(unlist(sapply(between_effects, function(i) {
+      grep(i, parameters$Parameter, fixed = TRUE)
+    })))
+    parameters$Component[index] <- "between"
+  }
+
+  interactions <- grep(":", parameters$Parameter, fixed = TRUE)
+  if (length(interactions)) {
+    parameters$Component[interactions] <- "interactions"
+  }
+
+  if (!("within" %in% parameters$Component) || !("between" %in% parameters$Component)) {
+    parameters$Component <- NULL
+  }
+
+  parameters
+}
+
+
+
+#' @importFrom stats model.frame
+.find_within_between <- function(model, which_effect) {
+  mf <- stats::model.frame(model)
+  unlist(sapply(names(mf), function(i) {
+    if (!is.null(attr(mf[[i]], which_effect, exact = TRUE))) {
+      i
+    }
+  }))
+}
+
+
+
+
 
 
 # Bayes function ------------------------------------------------------
@@ -225,20 +385,13 @@
 
 #' @importFrom stats sd setNames
 #' @keywords internal
-.extract_parameters_bayesian <- function(model, centrality = "median", dispersion = FALSE, ci = .89, ci_method = "hdi", test = c("pd", "rope"), rope_range = "default", rope_ci = 1.0, bf_prior = NULL, diagnostic = c("ESS", "Rhat"), priors = TRUE, iterations = 1000, ...) {
+.extract_parameters_bayesian <- function(model, centrality = "median", dispersion = FALSE, ci = .89, ci_method = "hdi", test = c("pd", "rope"), rope_range = "default", rope_ci = 1.0, bf_prior = NULL, diagnostic = c("ESS", "Rhat"), priors = TRUE, ...) {
 
   # MCMCglmm need special handling
   if (inherits(model, "MCMCglmm")) {
     parameters <- bayestestR::describe_posterior(model, centrality = centrality, dispersion = dispersion, ci = ci, ci_method = ci_method, test = test, rope_range = rope_range, rope_ci = rope_ci, diagnostic = "ESS", ...)
-
-    # Bayesian Models
-  } else if ((insight::is_multivariate(model) && insight::model_info(model)[[1]]$is_bayesian) || insight::model_info(model)$is_bayesian) {
-    parameters <- bayestestR::describe_posterior(model, centrality = centrality, dispersion = dispersion, ci = ci, ci_method = ci_method, test = test, rope_range = rope_range, rope_ci = rope_ci, bf_prior = bf_prior, diagnostic = diagnostic, priors = priors, ...)
-
-    # Bootstrapped Models
   } else {
-    data <- bootstrap_model(model, iterations = iterations)
-    parameters <- bayestestR::describe_posterior(data, centrality = centrality, dispersion = dispersion, ci = ci, ci_method = ci_method, test = test, rope_range = rope_range, rope_ci = rope_ci, bf_prior = bf_prior, ...)
+    parameters <- bayestestR::describe_posterior(model, centrality = centrality, dispersion = dispersion, ci = ci, ci_method = ci_method, test = test, rope_range = rope_range, rope_ci = rope_ci, bf_prior = bf_prior, diagnostic = diagnostic, priors = priors, ...)
   }
 
   if (length(ci) > 1) {
@@ -496,7 +649,7 @@
         "Parameter2" = names[2],
         "Mean_Parameter1" = model$estimate[1],
         "Mean_Parameter2" = model$estimate[2],
-        "Difference" = model$estimate[2] - model$estimate[1],
+        "Difference" = model$estimate[1] - model$estimate[2],
         "t" = model$statistic,
         "df" = model$parameter,
         "p" = model$p.value,
