@@ -4,13 +4,14 @@
 #'
 #' @param model Model object.
 #' @param ci Confidence Interval (CI) level. Default to 0.95 (95\%).
-#' @param bootstrap Should estimates be based on bootstrapped model? If \code{TRUE}, then arguments of \link[=model_parameters.stanreg]{Bayesian regressions} apply (see also \code{\link[=parameters_bootstrap]{parameters_bootstrap()}}).
+#' @param bootstrap Should estimates be based on bootstrapped model? If \code{TRUE}, then arguments of \link[=model_parameters.stanreg]{Bayesian regressions} apply (see also \code{\link[=bootstrap_parameters]{bootstrap_parameters()}}).
 #' @param iterations The number of bootstrap replicates. This only apply in the case of bootstrapped frequentist models.
-#' @param standardize The method used for standardizing the parameters. Can be \code{"refit"}, \code{"posthoc"}, \code{"smart"}, \code{"basic"} or \code{NULL} (default) for no standardization. See 'Details' in \code{\link[effectsize]{standardize_parameters}}.
+#' @param standardize The method used for standardizing the parameters. Can be \code{"refit"}, \code{"posthoc"}, \code{"smart"}, \code{"basic"} or \code{NULL} (default) for no standardization. See 'Details' in \code{\link[effectsize]{standardize_parameters}}. Note that robust estimation (i.e. \code{robust=TRUE}) of standardized parameters only works when \code{standardize="refit"}.
 #' @param exponentiate Logical, indicating whether or not to exponentiate the the coefficients (and related confidence intervals). This is typical for, say, logistic regressions, or more generally speaking: for models with log or logit link.
-#' @param robust Logical, if \code{TRUE}, robust standard errors are calculated (if possible), and confidence intervals and p-values are based on these robust standard errors.
+#' @param robust Logical, if \code{TRUE}, robust standard errors are calculated (if possible), and confidence intervals and p-values are based on these robust standard errors. Additional arguments like \code{vcov_estimation} or \code{vcov_type} are passed down to other methods, see \code{\link[=standard_error_robust]{standard_error_robust()}} for details.
 #' @param component Model component for which parameters should be shown. May be one of \code{"conditional"}, \code{"precision"} (\pkg{betareg}), \code{"scale"} (\pkg{ordinal}), \code{"extra"} (\pkg{glmx}) or \code{"all"}.
-#' @param ... Arguments passed to or from other methods.
+#' @param p_adjust Character vector, if not \code{NULL}, indicates the method to adjust p-values. See \code{\link[stats]{p.adjust}} for details.
+#' @param ... Arguments passed to or from other methods. For instance, when \code{bootstrap = TRUE}, arguments like \code{ci_method} are passed down to \code{\link[bayestestR]{describe_posterior}}.
 #'
 #' @seealso \code{\link[=standardize_names]{standardize_names()}} to rename
 #'   columns into a consistent, standardized naming scheme.
@@ -34,9 +35,12 @@
 #' # logistic regression model
 #' model <- glm(vs ~ wt + cyl, data = mtcars, family = "binomial")
 #' model_parameters(model)
+#'
+#' # show odds ratio / exponentiated coefficients
+#' model_parameters(model, exponentiate = TRUE)
 #' @return A data frame of indices related to the model's parameters.
 #' @export
-model_parameters.default <- function(model, ci = .95, bootstrap = FALSE, iterations = 1000, standardize = NULL, exponentiate = FALSE, robust = FALSE, ...) {
+model_parameters.default <- function(model, ci = .95, bootstrap = FALSE, iterations = 1000, standardize = NULL, exponentiate = FALSE, robust = FALSE, p_adjust = NULL, ...) {
   out <- .model_parameters_generic(
     model = model,
     ci = ci,
@@ -47,6 +51,7 @@ model_parameters.default <- function(model, ci = .95, bootstrap = FALSE, iterati
     exponentiate = exponentiate,
     effects = "fixed",
     robust = robust,
+    p_adjust = p_adjust,
     ...
   )
 
@@ -56,7 +61,7 @@ model_parameters.default <- function(model, ci = .95, bootstrap = FALSE, iterati
 
 
 
-.model_parameters_generic <- function(model, ci = .95, bootstrap = FALSE, iterations = 1000, merge_by = "Parameter", standardize = NULL, exponentiate = FALSE, effects = "fixed", robust = FALSE, ...) {
+.model_parameters_generic <- function(model, ci = .95, bootstrap = FALSE, iterations = 1000, merge_by = "Parameter", standardize = NULL, exponentiate = FALSE, effects = "fixed", robust = FALSE, df_method = NULL, p_adjust = NULL, ...) {
   # to avoid "match multiple argument error", check if "component" was
   # already used as argument and passed via "...".
   mc <- match.call()
@@ -67,9 +72,9 @@ model_parameters.default <- function(model, ci = .95, bootstrap = FALSE, iterati
     parameters <- bootstrap_parameters(model, iterations = iterations, ci = ci, ...)
   } else {
     parameters <- if (is.null(comp_argument)) {
-      .extract_parameters_generic(model, ci = ci, component = "conditional", merge_by = merge_by, standardize = standardize, effects = effects, robust = robust, ...)
+      .extract_parameters_generic(model, ci = ci, component = "conditional", merge_by = merge_by, standardize = standardize, effects = effects, robust = robust, df_method = df_method, p_adjust = p_adjust, ...)
     } else {
-      .extract_parameters_generic(model, ci = ci, merge_by = merge_by, standardize = standardize, effects = effects, robust = robust, ...)
+      .extract_parameters_generic(model, ci = ci, merge_by = merge_by, standardize = standardize, effects = effects, robust = robust, df_method = df_method, p_adjust = p_adjust, ...)
     }
   }
 
@@ -86,30 +91,9 @@ model_parameters.default <- function(model, ci = .95, bootstrap = FALSE, iterati
 # other special cases ------------------------------------------------
 
 
-#' @rdname model_parameters.merMod
-#' @export
-model_parameters.mixor <- function(model, ci = .95, effects = c("all", "fixed", "random"), bootstrap = FALSE, iterations = 1000, standardize = NULL, exponentiate = FALSE, ...) {
-  effects <- match.arg(effects)
-  out <- .model_parameters_generic(
-    model = model,
-    ci = ci,
-    bootstrap = bootstrap,
-    iterations = iterations,
-    merge_by = c("Parameter", "Effects"),
-    standardize = standardize,
-    exponentiate = exponentiate,
-    effects = effects,
-    ...
-  )
-
-  attr(out, "object_name") <- deparse(substitute(model), width.cutoff = 500)
-  out
-}
-
-
 #' @rdname model_parameters.default
 #' @export
-model_parameters.betareg <- function(model, ci = .95, bootstrap = FALSE, iterations = 1000, component = c("conditional", "precision", "all"), standardize = NULL, exponentiate = FALSE, ...) {
+model_parameters.betareg <- function(model, ci = .95, bootstrap = FALSE, iterations = 1000, component = c("conditional", "precision", "all"), standardize = NULL, exponentiate = FALSE, p_adjust = NULL, ...) {
   component <- match.arg(component)
   if (component == "all") {
     merge_by <- c("Parameter", "Component")
@@ -128,6 +112,8 @@ model_parameters.betareg <- function(model, ci = .95, bootstrap = FALSE, iterati
     merge_by = c("Parameter", "Component"),
     standardize = standardize,
     exponentiate = exponentiate,
+    robust = FALSE,
+    p_adjust = p_adjust,
     ...
   )
 
@@ -139,7 +125,7 @@ model_parameters.betareg <- function(model, ci = .95, bootstrap = FALSE, iterati
 
 #' @rdname model_parameters.default
 #' @export
-model_parameters.clm2 <- function(model, ci = .95, bootstrap = FALSE, iterations = 1000, component = c("all", "conditional", "scale"), standardize = NULL, exponentiate = FALSE, ...) {
+model_parameters.clm2 <- function(model, ci = .95, bootstrap = FALSE, iterations = 1000, component = c("all", "conditional", "scale"), standardize = NULL, exponentiate = FALSE, p_adjust = NULL, ...) {
   component <- match.arg(component)
   if (component == "all") {
     merge_by <- c("Parameter", "Component")
@@ -158,6 +144,8 @@ model_parameters.clm2 <- function(model, ci = .95, bootstrap = FALSE, iterations
     merge_by = c("Parameter", "Component"),
     standardize = standardize,
     exponentiate = exponentiate,
+    robust = FALSE,
+    p_adjust = p_adjust,
     ...
   )
 
@@ -172,7 +160,7 @@ model_parameters.clmm2 <- model_parameters.clm2
 
 #' @rdname model_parameters.default
 #' @export
-model_parameters.glmx <- function(model, ci = .95, bootstrap = FALSE, iterations = 1000, component = c("all", "conditional", "extra"), standardize = NULL, exponentiate = FALSE, ...) {
+model_parameters.glmx <- function(model, ci = .95, bootstrap = FALSE, iterations = 1000, component = c("all", "conditional", "extra"), standardize = NULL, exponentiate = FALSE, p_adjust = NULL, ...) {
   component <- match.arg(component)
   if (component == "all") {
     merge_by <- c("Parameter", "Component")
@@ -189,98 +177,11 @@ model_parameters.glmx <- function(model, ci = .95, bootstrap = FALSE, iterations
     merge_by = merge_by,
     standardize = standardize,
     exponentiate = exponentiate,
+    robust = FALSE,
+    p_adjust = p_adjust,
     ...
   )
 
   attr(out, "object_name") <- deparse(substitute(model), width.cutoff = 500)
   out
 }
-
-
-
-#' @export
-model_parameters.mlm <- function(model, ci = .95, bootstrap = FALSE, iterations = 1000, standardize = NULL, exponentiate = FALSE, ...) {
-  out <- .model_parameters_generic(
-    model = model,
-    ci = ci,
-    bootstrap = bootstrap,
-    iterations = iterations,
-    merge_by = c("Parameter", "Response"),
-    standardize = standardize,
-    exponentiate = exponentiate,
-    ...
-  )
-
-  attr(out, "object_name") <- deparse(substitute(model), width.cutoff = 500)
-  out
-}
-
-
-#' @export
-model_parameters.multinom <- model_parameters.mlm
-
-
-#' @export
-model_parameters.brmultinom <- model_parameters.mlm
-
-
-#' @export
-model_parameters.bracl <- model_parameters.mlm
-
-
-#' @importFrom stats qt setNames
-#' @export
-model_parameters.rma <- function(model, ci = .95, bootstrap = FALSE, iterations = 1000, standardize = NULL, exponentiate = FALSE, ...) {
-  meta_analysis_overall <- .model_parameters_generic(
-    model = model,
-    ci = ci,
-    bootstrap = bootstrap,
-    iterations = iterations,
-    merge_by = "Parameter",
-    standardize = standardize,
-    exponentiate = exponentiate,
-    ...
-  )
-
-  alpha <- (1 + ci) / 2
-
-  rma_parameters <- if (!is.null(model$slab)) {
-    sprintf("Study %s", model$slab)
-  } else {
-    sprintf("Study %i", 1:model[["k"]])
-  }
-
-  rma_coeffients <- as.vector(model$yi)
-  rma_se <- as.vector(sqrt(model$vi))
-  rma_ci_low <- rma_coeffients - rma_se * stats::qt(alpha, df = Inf)
-  rma_ci_high <- rma_coeffients + rma_se * stats::qt(alpha, df = Inf)
-  rma_statistic <- rma_coeffients / rma_se
-  rma_ci_p <- 2 * stats::pt(abs(rma_statistic), df = Inf, lower.tail = FALSE)
-
-  meta_analysis_studies <- data.frame(
-    Parameter = rma_parameters,
-    Coefficient = rma_coeffients,
-    SE = rma_se,
-    CI_low = rma_ci_low,
-    CI_high = rma_ci_high,
-    z = rma_statistic,
-    df_error = NA,
-    p = rma_ci_p,
-    Weight = 1 / as.vector(model$vi),
-    stringsAsFactors = FALSE
-  )
-
-  original_attributes <- attributes(meta_analysis_overall)
-  out <- merge(meta_analysis_studies, meta_analysis_overall, all = TRUE, sort = FALSE)
-
-  original_attributes$names <- names(out)
-  original_attributes$row.names <- 1:nrow(out)
-  original_attributes$pretty_names <- stats::setNames(out$Parameter, out$Parameter)
-  attributes(out) <- original_attributes
-
-  # no df
-  out$df_error <- NULL
-  attr(out, "object_name") <- deparse(substitute(model), width.cutoff = 500)
-
-  out
-}
