@@ -1,13 +1,13 @@
 #' Principal Component Analysis (PCA)
 #'
-#' This function performs a principal component analysis (PCA) and returns the loadings as a dataframe.
+#' This function performs a principal component analysis (PCA) and returns the loadings as a data frame.
 #'
-#' @param x A dataframe or a statistical model.
+#' @param x A data frame or a statistical model.
 #' @param n Number of components to extract. If \code{n="all"}, then \code{n} is set as the number of variables minus 1 (\code{ncol(x)-1}). If \code{n="auto"} (default) or \code{n=NULL}, the number of components is selected through \code{\link{n_factors}}. In \code{\link{reduce_parameters}}, can also be \code{"max"}, in which case it will select all the components that are maximally pseudo-loaded (i.e., correlated) by at least one variable.
-#' @param rotation If not \code{"none"}, the PCA will be computed using the \pkg{psych} package. Possible options include \code{"varimax"}, \code{"quartimax"}, \code{"promax"}, \code{"oblimin"}, \code{"simplimax"}, and \code{"cluster"}. See \code{\link[psych]{fa}} for details.
+#' @param rotation If not \code{"none"}, the PCA / FA will be computed using the \pkg{psych} package. Possible options include \code{"varimax"}, \code{"quartimax"}, \code{"promax"}, \code{"oblimin"}, \code{"simplimax"}, and \code{"cluster"}. See \code{\link[psych]{fa}} for details.
 #' @param sort Sort the loadings.
 #' @param threshold A value between 0 and 1 indicates which (absolute) values from the loadings should be removed. An integer higher than 1 indicates the n strongest loadings to retain. Can also be \code{"max"}, in which case it will only display the maximum loading per variable (the most simple structure).
-#' @param standardize A logical value indicating whether the variables should be standardized (centred and scaled) to have unit variance before the analysis takes place (in general, such scaling is advisable).
+#' @param standardize A logical value indicating whether the variables should be standardized (centered and scaled) to have unit variance before the analysis takes place (in general, such scaling is advisable).
 #' @param ... Arguments passed to or from other methods.
 #'
 #' @details
@@ -43,7 +43,9 @@
 #'  (Source: \href{https://stats.stackexchange.com/q/1576/54740}{CrossValidated})
 #'  }
 #'
-#' @note There is a \code{summary()}-method that prints the Eigenvalues and (explained) variance for each extracted component.
+#' @note There is a \code{summary()}-method that prints the Eigenvalues and (explained) variance for each extracted component. \code{closest_component()} will return a numeric vector with the assigned component index for each column from the original data frame. There is also a \href{https://easystats.github.io/see/articles/parameters.html}{\code{plot()}-method} implemented in the \href{https://easystats.github.io/see/}{\pkg{see}-package}.
+#'
+#' @seealso \code{\link[performance]{check_itemscale}} to compute various measures of internal consistencies applied to the (sub)scales (i.e. components) extracted from the PCA.
 #'
 #' @examples
 #' library(parameters)
@@ -52,9 +54,13 @@
 #' principal_components(mtcars[, 1:7], n = 2, rotation = "oblimin", threshold = "max", sort = TRUE)
 #' principal_components(mtcars[, 1:7], n = 2, threshold = 2, sort = TRUE)
 #'
-#' pca <- principal_components(mtcars[, 1:5], n = 2)
+#' pca <- principal_components(mtcars[, 1:5], n = 2, rotation = "varimax")
 #' summary(pca)
 #' predict(pca)
+#'
+#' # which variables from the original data belong to which extracted component?
+#' closest_component(pca)
+#'
 #' \donttest{
 #' # Automated number of components
 #' principal_components(mtcars[, 1:4], n = "auto")
@@ -73,14 +79,23 @@ principal_components <- function(x, n = "auto", rotation = "none", sort = FALSE,
   UseMethod("principal_components")
 }
 
+#' @rdname principal_components
+#' @export
+closest_component <- function(x) {
+  attributes(x)$closest_component
+}
 
 
-#' @importFrom stats prcomp na.omit
+
+#' @importFrom stats prcomp na.omit setNames
 #' @export
 principal_components.data.frame <- function(x, n = "auto", rotation = "none", sort = FALSE, threshold = NULL, standardize = TRUE, ...) {
   # save name of data set
   data_name <- deparse(substitute(x))
 
+  # original data
+  original_data <- x
+
   # remove missings
   x <- stats::na.omit(x)
 
@@ -93,7 +108,7 @@ principal_components.data.frame <- function(x, n = "auto", rotation = "none", so
 
   # Rotation
   if (rotation != "none") {
-    loadings <- .pca_rotate(x, n, rotation = rotation, sort = sort, threshold = threshold, ...)
+    loadings <- .pca_rotate(x, n, rotation = rotation, sort = sort, threshold = threshold, original_data = original_data, ...)
     attr(loadings, "data") <- data_name
     return(loadings)
   }
@@ -161,7 +176,12 @@ principal_components.data.frame <- function(x, n = "auto", rotation = "none", so
 
   # Add some more attributes
   attr(loadings, "loadings_long") <- .long_loadings(loadings, threshold = threshold)
+  # here we match the original columns in the data set with the assigned components
+  # for each variable, so we know which column in the original data set belongs
+  # to which extracted component...
+  attr(loadings, "closest_component") <- .closest_component(loadings, loadings_columns = loading_cols, variable_names = colnames(x))
   attr(loadings, "data") <- data_name
+  attr(loadings, "data_set") <- original_data
 
   # add class-attribute for printing
   class(loadings) <- unique(c("parameters_pca", "see_parameters_pca", class(loadings)))
@@ -192,7 +212,7 @@ principal_components.data.frame <- function(x, n = "auto", rotation = "none", so
 
 
 #' @keywords internal
-.pca_rotate <- function(x, n, rotation, sort = FALSE, threshold = NULL, ...) {
+.pca_rotate <- function(x, n, rotation, sort = FALSE, threshold = NULL, original_data = NULL, ...) {
   if (!(rotation %in% c("varimax", "quartimax", "promax", "oblimin", "simplimax", "cluster", "none"))) {
     stop("`rotation` must be one of \"varimax\", \"quartimax\", \"promax\", \"oblimin\", \"simplimax\", \"cluster\" or \"none\".")
   }
@@ -214,5 +234,15 @@ principal_components.data.frame <- function(x, n = "auto", rotation = "none", so
   msa <- psych::KMO(x)
 
   attr(pca, "MSA") <- msa$MSAi
-  model_parameters(pca, sort = sort, threshold = threshold)
+  out <- model_parameters(pca, sort = sort, threshold = threshold)
+
+  attr(out, "data_set") <- original_data
+  out
 }
+
+
+
+.closest_component <- function(loadings, loadings_columns, variable_names) {
+  component_columns <- apply(loadings[loadings_columns], 1, function(i) which.max(abs(i)))
+  stats::setNames(component_columns, variable_names)
+}
\ No newline at end of file
