@@ -11,22 +11,14 @@
 #' @param method If \code{"robust"}, robust standard errors are computed
 #'   by calling \code{\link[=standard_error_robust]{standard_error_robust()}}.
 #'   \code{standard_error_robust()}, in turn, calls one of the \code{vcov*()}-functions
-#'   from the \pkg{sandwich}-package for robust covariance matrix estimators.
-#' @param vcov_estimation String, indicating the suffix of the \code{vcov*()}-function
-#'   from the \pkg{sandwich}-package, e.g. \code{vcov_estimation = "CL"} (which
-#'   calls \code{\link[sandwich]{vcovCL}} to compute clustered covariance matrix
-#'   estimators), or \code{vcov_estimation = "HC"} (which calls
-#'   \code{\link[sandwich]{vcovHC}} to compute heteroskedasticity-consistent
-#'   covariance matrix estimators).
-#' @param vcov_type Character vector, specifying the estimation type for the
-#'   robust covariance matrix estimation (see \code{\link[sandwich]{vcovHC}} for
-#'   details).
-#' @param vcov_args List of named vectors, used as additional arguments that
-#'   are passed down to the \pkg{sandwich}-function specified in \code{vcov_estimation}.
+#'   from the \pkg{sandwich} or \pkg{clubSandwich} package for robust covariance
+#'   matrix estimators. For certain mixed models, \code{method} may also be one
+#'   of \code{"wald"}, \code{\link[=p_value_ml1]{"ml1"}}, \code{\link[=p_value_betwithin]{"betwithin"}},
+#'   \code{\link[=p_value_satterthwaite]{"satterthwaite"}} or \code{\link[=p_value_kenward]{"kenward"}}.
 #' @param verbose Toggle off warnings.
 #' @param ... Arguments passed to or from other methods. For \code{standard_error()},
 #'   if \code{method = "robust"}, arguments \code{vcov_estimation}, \code{vcov_type}
-#'   and \code{vcov_args} can be passed down to \code{standard_error_robust()}.
+#'   and \code{vcov_args} can be passed down to \code{\link[=standard_error_robust]{standard_error_robust()}}.
 #' @param effects Should standard errors for fixed effects or random effects
 #'    be returned? Only applies to mixed models. May be abbreviated. When
 #'    standard errors for random effects are requested, for each grouping factor
@@ -34,24 +26,11 @@
 #'    is returned.
 #' @inheritParams simulate_model
 #'
-#' @note \code{standard_error_robust()} resp. \code{standard_error(method = "robust")}
-#'   rely on the \pkg{sandwich}-package and will thus only work for those models
-#'   supported by that package.
+#' @note For Bayesian models (from \pkg{rstanarm} or \pkg{brms}), the standard error is the SD of the posterior samples.
 #'
 #' @examples
 #' model <- lm(Petal.Length ~ Sepal.Length * Species, data = iris)
 #' standard_error(model)
-#'
-#' # robust standard errors, calling sandwich::vcovHC(type="HC3") by default
-#' standard_error_robust(model)
-#'
-#' # cluster-robust standard errors, using clubSandwich
-#' iris$cluster <- factor(rep(LETTERS[1:8], length.out = nrow(iris)))
-#' standard_error_robust(
-#'   model,
-#'   vcov_type = "CR2",
-#'   vcov_args = list(cluster = iris$cluster)
-#' )
 #' @return A data frame.
 #' @importFrom stats coef vcov setNames var na.omit
 #' @importFrom insight get_varcov print_color get_parameters find_parameters
@@ -152,6 +131,16 @@ standard_error.effectsize_std_params <- function(model, ...) {
 }
 
 
+#' @export
+standard_error.parameters_skewness <- function(model, ...) {
+  attributes(model)$SE
+}
+
+#' @export
+standard_error.parameters_kurtosis <- standard_error.parameters_skewness
+
+
+
 
 
 
@@ -163,10 +152,18 @@ standard_error.effectsize_std_params <- function(model, ...) {
 #' @rdname standard_error
 #' @export
 standard_error.default <- function(model, method = NULL, ...) {
-  robust <- !is.null(method) && method == "robust"
+  if (!is.null(method)) {
+    method <- tolower(method)
+  } else {
+    method <- "wald"
+  }
 
-  if (isTRUE(robust)) {
+  if (method == "robust") {
     standard_error_robust(model, ...)
+  } else if (method == "ml1") {
+    se_ml1(model)
+  } else if (method == "betwithin") {
+    se_betwithin(model)
   } else {
     se <- tryCatch(
       {
@@ -189,8 +186,9 @@ standard_error.default <- function(model, method = NULL, ...) {
       se <- tryCatch(
         {
           varcov <- insight::get_varcov(model)
-          se <- sqrt(diag(varcov))
-          names(se) <- colnames(varcov)
+          se_from_varcov <- sqrt(diag(varcov))
+          names(se_from_varcov) <- colnames(varcov)
+          se_from_varcov
         },
         error = function(e) {
           NULL
@@ -303,6 +301,7 @@ standard_error.glm <- standard_error.lm
 #' @export
 standard_error.merMod <- function(model, effects = c("fixed", "random"), method = NULL, ...) {
   effects <- match.arg(effects)
+  if (is.null(method)) method <- "wald"
   robust <- !is.null(method) && method == "robust"
 
   if (effects == "random") {
@@ -332,10 +331,21 @@ standard_error.merMod <- function(model, effects = c("fixed", "random"), method
     if (isTRUE(robust)) {
       standard_error_robust(model, ...)
     } else {
-      .data_frame(
-        Parameter = insight::find_parameters(model, effects = "fixed", component = "conditional", flatten = TRUE),
-        SE = .get_se_from_summary(model)
-      )
+      # Classic and Satterthwaite SE
+      if (method %in% c("wald", "satterthwaite")) {
+        .data_frame(
+          Parameter = insight::find_parameters(model, effects = "fixed", component = "conditional", flatten = TRUE),
+          SE = .get_se_from_summary(model)
+        )
+        # ml1 approx
+      } else if (method == "ml1") {
+        se_ml1(model)
+      } else if (method == "betwithin") {
+        se_betwithin(model)
+        # Kenward approx
+      } else if (method %in% c("kenward", "kr")) {
+        se_kenward(model)
+      }
     }
   }
 }
@@ -447,19 +457,34 @@ standard_error.MixMod <- function(model, effects = c("fixed", "random"), compone
 # Zero-inflated models --------------------------------------------------------
 
 
+#' @rdname standard_error
 #' @export
-standard_error.zeroinfl <- function(model, component = c("all", "conditional", "zi", "zero_inflated"), ...) {
+standard_error.zeroinfl <- function(model, component = c("all", "conditional", "zi", "zero_inflated"), method = NULL, ...) {
   component <- match.arg(component)
   if (is.null(.check_component(model, component))) {
     return(NULL)
   }
 
+  robust <- !is.null(method) && method == "robust"
+  if (isTRUE(robust)) {
+    return(standard_error_robust(model, ...))
+  }
+
   cs <- .compact_list(stats::coef(summary(model)))
   x <- lapply(names(cs), function(i) {
     comp <- ifelse(i == "count", "conditional", "zi")
+
+    stats <- cs[[i]]
+
+    # remove log(theta)
+    theta <- grepl("Log(theta)", rownames(stats), fixed = TRUE)
+    if (any(theta)) {
+      stats <- stats[!theta, ]
+    }
+
     .data_frame(
       Parameter = insight::find_parameters(model, effects = "fixed", component = comp, flatten = TRUE),
-      SE = as.vector(cs[[i]][, 2]),
+      SE = as.vector(stats[, 2]),
       Component = comp
     )
   })
@@ -564,13 +589,20 @@ standard_error.coxme <- function(model, ...) {
 
 
 
+#' @rdname standard_error
 #' @export
-standard_error.coxph <- function(model, ...) {
+standard_error.coxph <- function(model, method = NULL, ...) {
+  robust <- !is.null(method) && method == "robust"
+  if (isTRUE(robust)) {
+    return(standard_error_robust(model, ...))
+  }
+
+  params <- insight::get_parameters(model)
   cs <- stats::coef(summary(model))
   se <- cs[, 3]
 
   .data_frame(
-    Parameter = .remove_backticks_from_string(names(se)),
+    Parameter = params$Parameter,
     SE = as.vector(se)
   )
 }
@@ -578,7 +610,12 @@ standard_error.coxph <- function(model, ...) {
 
 
 #' @export
-standard_error.survreg <- function(model, ...) {
+standard_error.survreg <- function(model, method = NULL, ...) {
+  robust <- !is.null(method) && method == "robust"
+  if (isTRUE(robust)) {
+    return(standard_error_robust(model, ...))
+  }
+
   s <- summary(model)
   se <- s$table[, 2]
 
@@ -673,7 +710,12 @@ standard_error.brmultinom <- standard_error.multinom
 
 
 #' @export
-standard_error.polr <- function(model, ...) {
+standard_error.polr <- function(model, method = NULL, ...) {
+  robust <- !is.null(method) && method == "robust"
+  if (isTRUE(robust)) {
+    return(standard_error_robust(model, ...))
+  }
+
   smry <- suppressMessages(as.data.frame(stats::coef(summary(model))))
   se <- smry[[2]]
   names(se) <- rownames(smry)
@@ -745,10 +787,52 @@ standard_error.bracl <- function(model, ...) {
 
 
 
+# Bayesian ----------------------------------------------
+
+
+#' @export
+standard_error.stanreg <- function(model, effects = c("fixed", "random"), component = c("all", "conditional", "zi", "zero_inflated"), ...) {
+  effects <- match.arg(effects)
+  component <- match.arg(component)
+
+  params <- insight::get_parameters(model, effects = effects, component = component, ...)
+
+  .data_frame(
+    Parameter = colnames(params),
+    SE = unname(sapply(params, stats::sd, na.rm = TRUE))
+  )
+}
+
+#' @export
+standard_error.brmsfit <- standard_error.stanreg
+
+#' @export
+standard_error.mvstanreg <- standard_error.stanreg
+
+
+
+
+
+
+
+
 
 # Other models ---------------------------------------------------------------
 
 
+#' @export
+standard_error.bife <- function(model, ...) {
+  cs <- summary(model)
+  se <- cs$cm[, 2]
+
+  .data_frame(
+    Parameter = .remove_backticks_from_string(rownames(cs$cm)),
+    SE = as.vector(se)
+  )
+}
+
+
+
 #' @export
 standard_error.cgam <- function(model, ...) {
   sc <- summary(model)
@@ -785,6 +869,39 @@ standard_error.cpglm <- function(model, ...) {
 
 
 
+#' @importFrom utils capture.output
+#' @export
+standard_error.zcpglm <- function(model, component = c("all", "conditional", "zi", "zero_inflated"), ...) {
+  if (!requireNamespace("cplm", quietly = TRUE)) {
+    stop("To use this function, please install package 'cplm'.")
+  }
+
+  component <- match.arg(component)
+  junk <- utils::capture.output(stats <- cplm::summary(model)$coefficients)
+  params <- get_parameters(model)
+
+  tweedie <- data.frame(
+    Parameter = params$Parameter[params$Component == "conditional"],
+    SE = as.vector(stats$tweedie[, "Std. Error"]),
+    Component = "conditional",
+    stringsAsFactors = FALSE,
+    row.names = NULL
+  )
+
+  zero <- data.frame(
+    Parameter = params$Parameter[params$Component == "zero_inflated"],
+    SE = as.vector(stats$zero[, "Std. Error"]),
+    Component = "zero_inflated",
+    stringsAsFactors = FALSE,
+    row.names = NULL
+  )
+
+  out <- .filter_component(rbind(tweedie, zero), component)
+  out
+}
+
+
+
 #' @export
 standard_error.cpglmm <- function(model, ...) {
   if (!requireNamespace("cplm", quietly = TRUE)) {
@@ -1018,6 +1135,7 @@ standard_error.psm <- standard_error.lrm
 
 
 
+#' @rdname standard_error
 #' @export
 standard_error.betareg <- function(model, component = c("all", "conditional", "precision"), ...) {
   component <- match.arg(component)
@@ -1040,6 +1158,32 @@ standard_error.betareg <- function(model, component = c("all", "conditional", "p
 }
 
 
+#' @rdname standard_error
+#' @export
+standard_error.DirichletRegModel <- function(model, component = c("all", "conditional", "precision"), ...) {
+  component <- match.arg(component)
+  params <- insight::get_parameters(model)
+
+  out <- .data_frame(
+    Parameter = params$Parameter,
+    Response = params$Response,
+    SE = as.vector(model$se)
+  )
+
+  if (!is.null(params$Component)) {
+    out$Component <- params$Component
+  } else {
+    component <- "all"
+  }
+
+  if (component != "all") {
+    out <- out[out$Component == component, ]
+  }
+
+  out
+}
+
+
 
 #' @export
 standard_error.gamlss <- function(model, ...) {
@@ -1210,6 +1354,22 @@ standard_error.rma <- function(model, ...) {
 
 
 
+#' @export
+standard_error.lavaan <- function(model, ...) {
+  out <- .extract_parameters_lavaan(model, ...)
+  out[out$Operator != "~1", c("To", "Operator", "From", "SE")]
+}
+
+
+
+#' @export
+standard_error.blavaan <- function(model, ci = .95, ...) {
+  out <- .extract_parameters_blavaan(model, ...)
+  out[out$Operator != "~1", c("To", "Operator", "From", "SE")]
+}
+
+
+
 
 
 
